// (c) Copyright IBM Corp. 2005, 2006 All rights reserved.
// This script is owned by International Business Machines Corporation or
// one of its subsidiaries ("IBM") and is copyrighted and licensed, not sold.
// You may copy, modify, and distribute this script in any form without
// payment to IBM, for any purpose including developing, using, marketing or
// distributing programs that include or are derivative works of the script.
// The script provided to you on an "AS IS" basis, without warranty of
// any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER
// EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE. IBM shall not be liable for indemnification or any damages you suffer 
// as a result of using, modifying or distributing the script or its derivatives.
// Each copy of any portion of this script or any derivative work, must
// include the above copyright notice and disclaimer of warranty.
/*
 * Setup a UDDI node for Common Criteria EAL4 testing
 * Set properties:
 *    turn off useAuthInfo
 *    turn off autoRegisterUsers
 *    turn off keygenSignatureRequired
 * Set policies: none apply
 * 
 * Assumes exactly one UDDI node is available and initialized.
 * 
 * Parameters: java SetupUDDINodeForEvaluatedConfig <adminUser> <adminPass>
 * Reads hostname, port from wsadmin.properties
 * 
 * To compile (assuming setupCmdLine):
 * javac -extdirs $WAS_EXT_DIRS SetupUDDINodeForEvaluatedConfig.java
 * 
 * To run (assuming setupCmdLine):
 * java -Djava.ext.dirs=$WAS_EXT_DIRS:$JAVA_HOME/jre/lib/ext -Dwas.profile=$USER_INSTALL_ROOT -classpath $WAS_CLASSPATH:. $CLIENTSOAP $CLIENTSAS SetupUDDINodeForEvaluatedConfig.java
 */

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.*;

import javax.management.ObjectName;

import com.ibm.uddi.v3.management.ConfigurationProperty;
import com.ibm.websphere.management.AdminClient;
import com.ibm.websphere.management.AdminClientFactory;

public class SetupUDDINodeForEvaluatedConfig {
	private AdminClient adminClient = null;
	private ObjectName mBean = null;

	public static void main(String args[]) {
		String user = "testusr";
		String pass = "testusrpwd";
		if(args.length != 2) {
			System.err.println("Use to update UDDI config for EAL4 evaluated configuration");
			System.err.println("Paramaters: username password");
			return;
		}
		user = args[0];
		pass = args[1];
		
		SetupUDDINodeForEvaluatedConfig uddi_Node = new SetupUDDINodeForEvaluatedConfig();
		if(uddi_Node.setup(user, pass)) {
			System.err.println("SETUP SUCCEEDED");
			return;
		} else {
			System.err.println("SETUP FAILED");
			return;
		}
	}

	public boolean setup(String user, String pass) {
		try {
			System.out.println("Connecting to MBean server");
			connectToMBeanServer(user, pass);
			if(adminClient==null)
				return false;
			System.out.println("Setting up MBean");
			if(!setupMBean())
				return false;
		} catch(Exception ex) {
			System.err.println("Error connecting to WAS: " + ex.getMessage());
			ex.printStackTrace();
			return false;
		}

		// set UDDI properties
		System.out.println("Setting UDDI properties");
		Properties props = new Properties();
		props.setProperty("useAuthInfo", "false");
		props.setProperty("autoRegisterUsers", "false");
		props.setProperty("keygenSignatureRequired", "false");

		Enumeration propsnames = props.propertyNames();
		while(propsnames.hasMoreElements()) {
			String propToUpdateName = (String)propsnames.nextElement();
			try {
				ConfigurationProperty propToUpdate = new ConfigurationProperty();
				propToUpdate.setId(propToUpdateName);
				String type = getProperty(propToUpdateName).getType();

				if(type.toString().equals("java.lang.String")) {
					propToUpdate.setStringValue(props.getProperty(propToUpdateName));
				}
				if(type.toString().equals("java.lang.Integer")) {
					propToUpdate.setIntegerValue(Integer.valueOf(props.getProperty(propToUpdateName)).intValue());
				}
				if(type.toString().equals("java.lang.Boolean")) {
					propToUpdate.setBooleanValue(Boolean.valueOf(props.getProperty(propToUpdateName)).booleanValue());
				}
				System.out.println("- updating " + propToUpdateName + "=" + props.getProperty(propToUpdateName));
				updateProperty(propToUpdate);
			} catch(Exception ex) {
				System.err.println("Error updating property " + propToUpdateName + ": " + ex.getMessage());
				ex.printStackTrace();
				return false;
			}
		}

		return true;
	}

	private boolean setupMBean() throws Exception {
		// Locate the Mbean
		System.out.println("- find UDDI node");
		ObjectName queryName = new ObjectName("WebSphere:type=UddiNode,*");
		Set s = adminClient.queryNames(queryName, null);
		Iterator iter = s.iterator();
		while(iter.hasNext()) {
			mBean = (ObjectName)iter.next();
		}
		if(mBean == null) {
			System.err.println("Error finding UDDI MBean");
			return false;
		}
		return true;
	}
	private void connectToMBeanServer(String user, String pass) throws Exception {
		System.out.println("- finding wsadmin.properties");
		
		String profileHome = System.getProperty("was.profile");
		if(profileHome==null) {
			System.err.println("was.profile not set.  Run setupCmdLine from your WAS profile and specify -Dwas.profile=$USER_INSTALL_ROOT");
			return;
		}
		profileHome=profileHome+"/properties/wsadmin.properties";
		System.out.println("- guessed location "+profileHome);
		
		Properties props = new Properties();
		try {
			InputStream is = new FileInputStream(profileHome);
			if(is != null) {	
				props.load(is);
			} else {
				System.err.println("Properties file " + profileHome + " couldn't be opened");
				props = null;
			}
		} catch(java.io.IOException ex) {
				System.err.println("Properties file " + profileHome + " couldn't be opened: " + ex.toString());
				ex.printStackTrace();
				props = null;
		}
		if(props==null) return;
		String type = props.getProperty("com.ibm.ws.scripting.connectionType");
		String server = props.getProperty("com.ibm.ws.scripting.host");
		String port = props.getProperty("com.ibm.ws.scripting.port");
		System.out.println("- using a "+type+" connection to "+server+":"+port);
		Properties adminProps = new Properties();
		adminProps.setProperty(AdminClient.CONNECTOR_TYPE, type);
		adminProps.setProperty(AdminClient.CONNECTOR_HOST, server);
		adminProps.setProperty(AdminClient.CONNECTOR_PORT, port);
		adminProps.setProperty(AdminClient.USERNAME, user);
		adminProps.setProperty(AdminClient.PASSWORD, pass);
		adminProps.setProperty(AdminClient.CONNECTOR_SECURITY_ENABLED, "true");

		System.out.println("- createAdminClient");
		adminClient = AdminClientFactory.createAdminClient(adminProps);
		System.out.println("- create configService");
	}

	private ConfigurationProperty getProperty(String id) throws Exception {
		String[] signature = new String[]{"java.lang.String"};
		Object[] params = new String[]{id};
		return (ConfigurationProperty)invoke("getProperty", signature, params);
	}
	private void updateProperty(ConfigurationProperty property) throws Exception {
		String[] signature = new String[]{"com.ibm.uddi.v3.management.ConfigurationProperty"};
		Object[] params = new Object[]{property};
		invoke("updateProperty", signature, params);
	}
	private Object invoke(String operation, String[] signature, Object[] params) throws Exception {
		// invoke operation
		return adminClient.invoke(mBean, operation, params, signature);
	}

}
